lcd_instance = None

def get_lcd():
    """Get static reference to LCD display."""

    global lcd_instance

    if lcd_instance is None:
        from . import arduino, rgb_lcd

        lcd_instance = rgb_lcd.rgb_lcd(arduino().connection)
        lcd_instance.setRGB(255, 255, 255)

    return lcd_instance

def clear():
    """Clear all text from the LCD screen and reset the cursor to the
    top-left corner of the screen.
    """
 
    get_lcd().clear()

def move_cursor(row, col):
    """Move the cursor to a specific location.

    Parameters:
      row (int):       Row on the LCD screen (range: [0, 1])
      col (int):       Column on the LCD screen (range: [0, 15])
    """
    get_lcd().moveCursor(col, row)

def print(value):
    """Print a value to the LCD screen.

    Parameters:
      value:    Any value, of any type.
    """
    get_lcd().printString(str(value))

def hsv(hue, saturation, value):
    """Change the LCD background colour by HSV value.

    Parameters:
      hue (float):           colour (between 0 and 1)
      saturation (float):    vibrance (between 0 and 1)
      value (int):           brightness (between 0 and 255)
    """
    import colorsys
    red, green, blue = colorsys.hsv_to_rgb(hue, saturation, value)
    rgb(red, green, blue)

def rgb(red, green, blue):
    """Change the LCD background colour by RGB value.

    Parameters:
      red (int):             amount of red to display (0–255)
      green (int):           amount of green to display (0–255)
      blue (int):            amount of blue to display (0–255)
    """
    get_lcd().setRGB(red, green, blue)



"""CLI for Arduino API wrapper.
"""

import click

from . import *


@click.group()
def cli():
    pass

@cli.group()
def analog():
    """Commands to interact with analog pins."""
    pass

@analog.command('read')
@click.argument('pin', type=int)
def aread(pin):
    """Read a value from an analog pin."""
    print(analog_read(pin))

@cli.group()
def digital():
    """Commands to interact with digital pins."""
    pass

@digital.command('read')
@click.argument('pin', type=int)
def dread(pin):
    """Read a value from a digital pin."""
    print(digital_read(pin))


lcd_instance = None

def get_lcd():
    global lcd_instance

    if lcd_instance is None:
        from . import rgb_lcd
        lcd_instance = rgb_lcd.rgb_lcd(arduino().connection)
        lcd_instance.setRGB(255, 255, 255)

    return lcd_instance


@cli.group()
def lcd():
    """Commands to interact with an RGB LCD screen."""

@lcd.command('print')
@click.argument('message')
def lcd_print_message(message):
    lcd_clear()
    lcd_print(message)

@lcd.command()
@click.argument('hue', type=int)
@click.argument('saturation', type=float, default=1)
@click.argument('value', type=float, default=1)
def hsv(hue, saturation, value):
    """Set LCD background colour by hue, saturation and value."""
    lcd_hsv(hue / 360, saturation, value)

@lcd.command()
@click.argument('red', type=int)
@click.argument('green', type=int)
@click.argument('blue', type=int)
def rgb(red, green, blue):
    lcd_rgb(red, green, blue)



"""Arduino API wrapper.

This code assumes that there is only one Arduino device connected.
"""

import nanpy


# Assuming there's only one Arduino connected to this computer, here it is.
a = None

def arduino():
    """Return Arduino API, instantiating if needed."""

    global a

    if a is None:
        port = None

        import sys

        if sys.platform == 'win32':
            #print('Check Windows Device Manager to see which COM the Arduino Uno has been assigned')
            #port = input('Please specify COM port (eg. COM10): ')
            ## code from http://leim.altervista.org/blog/detect-arduino-board-in-python/?doing_wp_cron=1590438741.3099739551544189453125
            import serial.tools.list_ports
            
            ports = list(serial.tools.list_ports.comports())
            Arduino_ports=[]
            for p in ports:
                if 'Arduino' in p.description:
                    Arduino_ports.append(p)
            if len(Arduino_ports)==0:
                print("no Arduino board detected")
                
            if len(Arduino_ports) > 1:
                print('Multiple Arduinos found - using the first')
            else:
                print("Arduino board detected")
             
            if len(Arduino_ports) >= 1:
              print(Arduino_ports[0].device)
              port = Arduino_ports[0].device

        elif sys.platform == 'darwin':
            # Find /dev/cu.usbmodem* using the glob module:
            import glob
            ports = glob.glob('/dev/cu.usbmodem*')

            if len(ports) == 0:
                sys.stderr.write('No Arduino device detected (plugged in?)\n')
                sys.exit(1)

            elif len(ports) == 1:
                port = ports[0]

            else:
                print('Multiple devices detected:')
                for i, name in enumerate(ports):
                    print(f'[{i}]    {name}')

                while port == None:
                    try:
                        i = int(input('select device> '))
                        port = ports[int(i)]

                    except IndexError as e:
                        sys.stderr.write('Invalid device number\n')

                    except ValueError as e:
                        print(e)

        a = nanpy.ArduinoApi(connection=nanpy.SerialManager(device=port))

    return a


def analog_read(pin):
    """Read a value from an analog port.

    Parameters:
      pin (int):     Analog pin to read from (e.g., 2 for A2).

    Returns:         Quantized analog value (0–1023).
    """
    return arduino().analogRead(pin+14)

def analog_write(pin, duty):
    """Write a value to an analog port.

    Parameters:
      pin (int):     Analog pin to write to (e.g., 2 for A2).
      duty (int):    Duty cycle to set (0–255).
    """
    return arduino().analogWrite(pin+14, duty)

def digital_read(pin):
    """Read a value from a digital port.

    Parameters:
      pin (int):     Digital pin to read from (e.g., 4 for D4).

    Returns:         True or False.
    """
    return arduino().digitalRead(pin)

def digital_write(pin, value):
    """Write a value to a digital port.

    Parameters:
      pin (int):     Digital pin to write to (e.g., 4 for D4).
      value (bool):  Value to write (True or False).
    """
    return arduino().digitalWrite(pin, value)


from . import lcd
lcd_clear = lcd.clear
lcd_move_cursor = lcd.move_cursor
lcd_print = lcd.print
lcd_hsv = lcd.hsv
lcd_rgb = lcd.rgb

from . import temperature
temp_celsius = temperature.celsius

from . import motor
servo_move = motor.move
servo_read = motor.read

from . import buzzer
buzzer_note = buzzer.note
buzzer_stop = buzzer.stop

from . import ir_distance
distance_cm = ir_distance.centimeters

from . import led_bar
led_bar_set_level = led_bar.set_led_level
led_bar_set_led_light = led_bar.set_led_light

from . import rgba_leds
rgba_leds_set_rgb = rgba_leds.set_led_rgb
rgba_leds_set_hsb = rgba_leds.set_led_hsb



from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import (arduinoobjectmethod, returns)

class led_bar(ArduinoObject):
    cfg_h_name = 'USE_led_bar'

    def __init__(self, pin, connection=None):
        ArduinoObject.__init__(self, connection=connection)
        self.id = self.call('new', pin)

    @arduinoobjectmethod
    def barSetLevel(self, value):
        pass

    @arduinoobjectmethod
    def barSetLed(self, led, value):
        pass
